@@ -4,7 +4,7 @@ import logging
 import aiohttp
 import asyncio
 
-from .exceptions import *
+from igwrapper.clients.exceptions import *
 
 from igwrapper.models import Position
 
@@ -17,9 +17,15 @@ class AIOIGRest:
         self.pwd = pwd
         self.key = key
         self.base_url = base_url
+        self._cst = None
+        self._xst = None
+        self._ls_url = None
+        self._active_account_id = None
 
     async def __aenter__(self):
         self.headers = await self._get_session_headers()
+        self._xst = self.headers['X-SECURITY-TOKEN']
+        self._cst = self.headers['CST']
         self._session = aiohttp.ClientSession(headers=self.headers, requote_redirect_url=False)
         return self
     
@@ -43,6 +49,11 @@ class AIOIGRest:
             async with client.post(url=url, headers=headers, json=payload) as r:
                 if r.status >= 400:
                     raise (IGException(f"Server problem: status code: {r.status}, reason: {r.reason}"))
+                self._xst = r.headers['X-SECURITY-TOKEN']
+                self._cst = r.headers['CST']
+                res = await r.json()
+                self._ls_url = res['lightstreamerEndpoint']
+                self._active_account_id = res['currentAccountId']
                 return {
                     **headers,
                     "X-SECURITY-TOKEN": r.headers['X-SECURITY-TOKEN'],
@@ -52,18 +63,19 @@ class AIOIGRest:
     @staticmethod
     def _api_limit_hit(rsp_txt):
         return 'exceeded-api-key-allowance' in rsp_txt or \
-               'exceeded-account-allowance' in rsp_txt or \
-               'exceeded-account-trading-allowance' in rsp_txt
+                'exceeded-account-allowance' in rsp_txt or \
+                'exceeded-account-trading-allowance' in rsp_txt or \
+                'exceeded-account-historical-data-allowance' in rsp_txt
 
     def parse_response(req):
         async def wrapper(*args, **kwargs):
             r = await req(*args, **kwargs)
-            if r.status >= 400:
-                raise (IGException(f"Server problem: status code: {r.status}, reason: {r.reason} for url: {r.url}"))
             text = await r.text()
             if AIOIGRest._api_limit_hit(text):
-                raise ApiExceededException()
+                raise ApiExceededException(f"API limit hit: {text}")
 
+            if r.status >= 400:
+                raise (IGException(f"Server problem: status code: {r.status}, reason: {r.reason} for url: {r.url}"))
             return await r.json()
 
         return wrapper
@@ -150,20 +162,25 @@ class AIOIGRest:
         # TODO fix this ting
         position_obj = position.get_opening_details()
 
-        for _ in range(1, attempts):
+        for i in range(1, attempts):
 
-            r = await self._req(action='post', url=url, payload=position_obj)
-            # Give small amount of time for ig systems to update
-            await asyncio.sleep(0.3)
+            try:
 
-            # TODO convert these to objects
-            confirm = await self.confirm_trade(r['dealReference'])
+                r = await self._req(action='post', url=url, payload=position_obj)
+                # Give small amount of time for ig systems to update
+                await asyncio.sleep(0.3)
 
-            if confirm['status'] == "OPEN":
-                log.debug("Opened position %s", position.epic)
-                position.is_open = True
-                position.level = confirm['level']
-                return position
+                # TODO convert these to objects
+                confirm = await self.confirm_trade(r['dealReference'])
+
+                if confirm['status'] == "OPEN":
+                    log.debug("Opened position %s", position.epic)
+                    position.is_open = True
+                    position.level = confirm['level']
+                    return position
+            except IGException as e:
+                log.error(f"Failed to open position for {position.epic}, attempt {i}")
+                continue
 
         log.error("Failed to open position: %s", position.epic)
         return
@@ -184,13 +201,13 @@ class AIOIGRest:
 
         if position is None: return
 
-        log.info("Closing position %s", position.epic)
-
         url = f'{self.base_url}/positions/otc'
 
+        # TODO bid here needs to change to level - anything that calls close_position
+        # should be handling what level to close at
         close_details = position.get_closing_details(position.bid - 0.1)
 
-        log.info("Closing position: %s", {close_details['epic']})
+        log.info("Closing position: %s", position.epic)
 
         for _ in range(1, attempts):
 
